#!/usr/bin/env python
# encoding: utf-8

from xml.etree import cElementTree as ET
from datetime import datetime
import optparse

"""
This script parses a projects XML data as exported by sourceforge
and outputs an XML as it is needed by launchpad for importing into
their trackers.

Released into the Public Domain by Holger Rapp <SirVer@gmx.de> in
2010.
"""

class NodeDict(dict):
    "A dictionary that gets it's data from sf.net XML nodes"

    def __init__(self, node = None):
        self.node = node

    def __getattr__(self, e):
        return self.__getitem__(e)

    def from_node(self, name, node,
                name_in_node = None, conversion_func = None):
        name_in_node = name_in_node or name

        val = node.find(name_in_node).text
        if conversion_func:
            nval = conversion_func(val)
            if nval is None:
                raise RuntimeError, "Conversion of %s failed!" % val
            val = nval
        self[name] = val

    def try_from_node(self, *args, **kwargs):
        try:
            self.from_node(*args, **kwargs)
        except AttributeError:
            pass

    def date_from_node(self, *args, **kwargs):
        return self.from_node(*args, conversion_func =
            lambda d: datetime.fromtimestamp(int(d)), **kwargs)

    def parse_lookup_dict(self, node, name, subname, idname, valuename):
        d = {}
        for n in node.find(name).findall(subname):
            d[n.find(idname).text] = n.find(valuename).text
        self[name] = d

def parse_tracker_items(ti_node, t):
    rv = []

    for item in ti_node:
        d = NodeDict()
        d.from_node("url", item)
        d.from_node("id", item)

        d.from_node("submitter", item)
        d.from_node("assignee", item)
        d.from_node("closer", item)
        d.date_from_node("submit_date", item)
        d.date_from_node("close_date", item)
        d.from_node("priority", item)
        d.from_node("summary", item)
        d.from_node("details", item)

        d.from_node("status", item, "status_id", t.statuses.get)
        d.from_node("category", item, "category_id", t.categories.get)
        d.from_node("group", item, "group_id", t.groups.get)
        d.try_from_node("resolution", item, "resolution_id", t.resolutions.get)

        # Parse follow ups
        d["followups"] = []
        for follow in item.find("followups"):
            f = NodeDict()
            f.from_node("id", follow)
            f.from_node("submitter", follow)
            f.date_from_node("date", follow)
            f.from_node("details", follow)

            d.followups.append(f)

        d["attachments"] = []
        for attach in item.find("attachments"):
            a = NodeDict()
            a.from_node("url", attach)
            a.from_node("id", attach)
            a.from_node("filename", attach)
            a.from_node("description", attach)
            a.from_node("filesize", attach)
            a.from_node("filetype", attach)
            a.date_from_node("date", attach)
            a.from_node("submitter", attach)

            d.attachments.append(a)

        # Parse history entries
        d["history_entries"] = []
        for hentry in item.find("history_entries"):
            f = NodeDict()
            f.from_node("id", hentry)
            f.from_node("field_name", hentry)
            f.date_from_node("date", hentry)
            f.from_node("old_value", hentry)
            f.try_from_node("new_value", hentry)
            f.from_node("updator", hentry)

            d.history_entries.append(f)

        rv.append(d)

    return rv


def parse_issues(tn):
    """
    Parse all sf.net trackers and returns a list of dictionaries which
    contain all elements from the issues in python data format.  Some
    resolving is done (for example resolve_id -> string)
    """
    issues = []
    for t in tn:
        # Tracker information
        tracker = NodeDict()
        tracker.from_node("url", t)
        tracker.from_node("name", t)

        tracker.parse_lookup_dict(t, "groups", "group", "id", "group_name")
        tracker.parse_lookup_dict(t, "categories", "category", "id", "category_name")
        tracker.parse_lookup_dict(t, "resolutions", "resolution", "id", "name")
        tracker.parse_lookup_dict(t, "statuses", "status", "id", "name")

        tracker.categories["100"] = "None"
        tracker.groups["100"] = "None"

        issues.extend(parse_tracker_items(t.find("tracker_items"), tracker))
    return issues



def main():
    p = optparse.OptionParser("%prog <sf.net xml project export file>")

    opts, args = p.parse_args()

    et = ET.parse(args[0])

    k = parse_issues(et.getroot().find("trackers"))
    print "len(k: %s" % (len(k))


if __name__ == '__main__':
    main()
